from typing import Iterable

from kafka import KafkaConsumer

from email_service.core.config import Config
from email_service.domain.email_schema import EmailRequest


class EmailConsumer:
    def __init__(self, config: Config, topic: str = "email") -> None:
        self.bootstrap_server = f"{config.kafka_host}:{config.kafka_port}"
        self.topic = topic
        self.client_id = f"email-service-{config.version}"
        self.consumer = self._setup_connection()

    def consume(self) -> Iterable[EmailRequest]:
        for message in self.consumer:
            yield EmailRequest.from_encoded_json(message.value)

    def _setup_connection(self) -> KafkaConsumer:
        return KafkaConsumer(
            self.topic,
            bootstrap_servers=self.bootstrap_server,
            client_id=self.client_id,
            enable_auto_commit=True,
            auto_offset_reset="latest",
        )



from kafka import KafkaProducer

from email_service.domain.email_schema import EmailRequest


class EmailPublisher:
    def __init__(self, host: str, port: int, topic: str = "email", key: str | None = None) -> None:
        self.host = host
        self.port = port
        self.topic = topic
        if key is None:
            self.key = None
        else:
            self.key = key.encode("utf-8")
        self.producer = self._setup_connection()

    def publish(self, email_request: EmailRequest) -> None:
        encoded_email = email_request.json().encode("utf-8")
        self.producer.send(topic=self.topic, key=self.key, value=encoded_email)
        self.producer.flush()

    def _setup_connection(self) -> KafkaProducer:
        return KafkaProducer(bootstrap_servers=f"{self.host}:{self.port}")



from pydantic import BaseSettings

from email_service.version import __version__


class Config(BaseSettings):
    version: str = __version__
    username: str
    password: str
    smtp_server: str
    smtp_port: int
    kafka_host: str
    kafka_port: int

    class Config:
        env_file = ".env"
        env_file_encoding = "utf-8"



from email_service.core.config import Config
from email_service.domain.consumer import EmailConsumer
from email_service.domain.email_server import SSLEmailServer
from email_service.domain.email_service import EmailService


def main() -> None:
    config = Config()
    email_server = SSLEmailServer(config=config)
    email_service = EmailService(email_server=email_server)
    consumer = EmailConsumer(config=config)
    for message in consumer.consume():
        email_service.send(request=message)


if __name__ == "__main__":
    main()



import logging
import sys

from email_service.domain.email_schema import EmailRequest
from email_service.domain.email_server import EmailServer

logging.basicConfig(stream=sys.stdout, level=logging.INFO)


class EmailService:
    def __init__(self, email_server: EmailServer) -> None:
        self.email_server = email_server

    def send(self, request: EmailRequest) -> None:
        self.email_server.send_email(message=request.message())
        logging.info(
            "'%s' from '%s' to '%s'",
            request.subject,
            request.from_address,
            request.to_address,
        )



from abc import ABC
from abc import abstractmethod
from email.message import EmailMessage
from smtplib import SMTP
from smtplib import SMTP_SSL

from email_service.core.config import Config


class EmailServer(ABC):
    def __init__(self, config: Config) -> None:
        self.config = config

    @abstractmethod
    def send_email(self, message: EmailMessage) -> None:
        raise NotImplementedError()


class DummyThiccEmailServer(EmailServer):
    def __init__(self, config: Config) -> None:
        super().__init__(config)
        self.messages: list[EmailMessage] = []

    def send_email(self, message: EmailMessage) -> None:
        self.messages.append(message)


class SSLEmailServer(EmailServer):
    def __init__(self, config: Config) -> None:
        super().__init__(config=config)
        self.server = SMTP_SSL(host=config.smtp_server, port=config.smtp_port, timeout=2)
        self._setup_connection()

    def send_email(self, message: EmailMessage) -> None:
        self.server.send_message(msg=message)

    def _setup_connection(self) -> None:
        self.server.login(user=self.config.username, password=self.config.password)


class StartTLSEmailServer(EmailServer):
    def __init__(self, config: Config) -> None:
        super().__init__(config=config)
        self.server = SMTP(host=config.smtp_server, port=config.smtp_port, timeout=2)

    def send_email(self, message: EmailMessage) -> None:
        self.server.send_message(msg=message)

    def _setup_connection(self) -> None:
        self.server.starttls()
        self.server.ehlo()
        self.server.login(user=self.config.username, password=self.config.password)



from distutils import util
from typing import Dict

import setuptools

version: Dict[str, str] = {}
path = util.convert_path("src/email_service/version.py")
with open(path) as version_file:
    exec(version_file.read(), version)  # noqa: S102 DUO105, WPS421

setuptools.setup(version=version["__version__"])



from email.headerregistry import Address
from email.message import EmailMessage
from email.utils import formatdate
import json
import mimetypes
from pathlib import Path

from pydantic import BaseModel


class EmailAddress(BaseModel):
    name: str
    email: str

    def to_address(self) -> Address:
        username, domain = self.email.split("@")
        return Address(display_name=self.name, domain=domain, username=username)


class Attachment(BaseModel):
    binary_data: bytes
    filename: str
    maintype: str
    subtype: str

    @classmethod
    def from_file(cls, attachment: Path) -> "Attachment":
        mimetype = mimetypes.types_map.get(attachment.suffix, "data/octet-stream")
        maintype, subtype = mimetype.split("/")
        with open(file=attachment, mode="rb") as fp:
            binary_data = fp.read()
        return cls(
            binary_data=binary_data,
            filename=attachment.name,
            maintype=maintype,
            subtype=subtype,
        )


class EmailRequest(BaseModel):
    from_address: EmailAddress
    to_address: EmailAddress
    subject: str
    text: str | None
    html: str | None
    bcc: list[EmailAddress] | None = None
    cc: list[EmailAddress] | None = None
    attachments: list[Attachment] = []

    @classmethod
    def from_encoded_json(cls, encoded_json: str) -> "EmailRequest":
        return cls(**json.loads(encoded_json))

    def message(self) -> EmailMessage:
        msg = EmailMessage()
        msg.set_content(self.text)
        if self.html:
            msg.add_alternative(self.html, subtype="html")
        msg["Subject"] = self.subject
        msg["From"] = self.from_address.to_address()
        msg["To"] = self.to_address.to_address()
        msg["Date"] = formatdate(localtime=True)
        if self.bcc:
            msg["Bcc"] = (bcc.to_address() for bcc in self.bcc)
        if self.cc:
            msg["Cc"] = (cc.to_address() for cc in self.cc)

        for attachment in self.attachments:
            msg.add_attachment(
                attachment.binary_data,
                maintype=attachment.maintype,
                subtype=attachment.subtype,
                filename=attachment.filename,
            )

        return msg
